@@ -1,19 +1,16 @@
 """
-    Commonly used dialogs
-    isort:skip_file
+Commonly used dialogs
+isort:skip_file
 """
-# Standard Library
-# pylint: disable=no-member
 import os
 from gettext import gettext as _
 
-# Third Party Libraries
 import gi
 gi.require_version("WebKit2", "4.0")
-from gi.repository import GLib, GObject, Gtk, WebKit2
+from gi.repository import GObject, Gtk, WebKit2
 
-# Lutris Modules
-from lutris import api, pga, runtime, settings
+from lutris import api, settings
+from lutris.database import sources as sources_db
 from lutris.gui.widgets.log_text_view import LogTextView
 from lutris.util import datapath
 from lutris.util.log import logger
@@ -39,6 +36,7 @@ class GtkBuilderDialog(GObject.Object):
     }
 
     def __init__(self, parent=None, **kwargs):
+        # pylint: disable=no-member
         super().__init__()
         ui_filename = os.path.join(datapath.get(), "ui", self.glade_file)
         if not os.path.exists(ui_filename):
@@ -128,13 +126,15 @@ class DirectoryDialog(Gtk.FileChooserDialog):
 
     """Ask the user to select a directory."""
 
-    def __init__(self, message, parent=None):
+    def __init__(self, message, default_path=None, parent=None):
         super().__init__(
             title=message,
             action=Gtk.FileChooserAction.SELECT_FOLDER,
             buttons=(_("_Cancel"), Gtk.ResponseType.CLOSE, _("_OK"), Gtk.ResponseType.OK),
             parent=parent,
         )
+        if default_path:
+            self.set_current_folder(default_path)
         self.result = self.run()
         self.folder = self.get_current_folder()
         self.destroy()
@@ -144,14 +144,18 @@ class FileDialog(Gtk.FileChooserDialog):
 
     """Ask the user to select a file."""
 
-    def __init__(self, message=None, default_path=None):
+    def __init__(self, message=None, default_path=None, mode="open"):
         self.filename = None
         if not message:
             message = _("Please choose a file")
+        if mode == "save":
+            action = Gtk.FileChooserAction.SAVE
+        else:
+            action = Gtk.FileChooserAction.OPEN
         super().__init__(
             message,
             None,
-            Gtk.FileChooserAction.OPEN,
+            action,
             (_("_Cancel"), Gtk.ResponseType.CANCEL, _("_OK"), Gtk.ResponseType.OK),
         )
         if default_path and os.path.exists(default_path):
@@ -167,7 +171,7 @@ class FileDialog(Gtk.FileChooserDialog):
 class InstallOrPlayDialog(Gtk.Dialog):
 
     def __init__(self, game_name):
-        Gtk.Dialog.__init__(self, "%s is already installed" % game_name)
+        Gtk.Dialog.__init__(self, _("%s is already installed") % game_name)
         self.connect("delete-event", lambda *x: self.destroy())
 
         self.action = "play"
@@ -203,34 +207,6 @@ class InstallOrPlayDialog(Gtk.Dialog):
         self.destroy()
 
 
-class RuntimeUpdateDialog(Gtk.Dialog):
-
-    """Dialog showing the progress of ongoing runtime update."""
-
-    def __init__(self, parent=None):
-        Gtk.Dialog.__init__(self, _("Runtime updating"), parent=parent)
-        self.set_size_request(360, 104)
-        self.set_border_width(12)
-        progress_box = Gtk.Box()
-        self.progressbar = Gtk.ProgressBar()
-        self.progressbar.set_margin_top(40)
-        self.progressbar.set_margin_bottom(40)
-        self.progressbar.set_margin_right(20)
-        self.progressbar.set_margin_left(20)
-        progress_box.pack_start(self.progressbar, True, True, 0)
-        self.get_content_area().add(progress_box)
-        GLib.timeout_add(200, self.on_runtime_check)
-        self.show_all()
-
-    def on_runtime_check(self, *args, **kwargs):  # pylint: disable=unused-argument
-        self.progressbar.pulse()
-        if not runtime.is_updating():
-            self.response(Gtk.ResponseType.OK)
-            self.destroy()
-            return False
-        return True
-
-
 class PgaSourceDialog(GtkBuilderDialog):
     glade_file = "dialog-pga-sources.ui"
     dialog_object = "pga_dialog"
@@ -250,7 +226,7 @@ class PgaSourceDialog(GtkBuilderDialog):
         uri_column = Gtk.TreeViewColumn("URI", renderer, text=0)
         self.sources_treeview.append_column(uri_column)
         self.sources_treeview.set_model(self.sources_liststore)
-        sources = pga.read_sources()
+        sources = sources_db.read_sources()
         for __, source in enumerate(sources):
             self.sources_liststore.append((source, ))
 
@@ -262,7 +238,7 @@ class PgaSourceDialog(GtkBuilderDialog):
         return [source[0] for source in self.sources_liststore]
 
     def on_apply(self, widget, data=None):
-        pga.write_sources(self.sources_list)
+        sources_db.write_sources(self.sources_list)
         self.on_close(widget, data)
 
     def on_add_source_button_clicked(self, widget, data=None):  # pylint: disable=unused-argument
@@ -340,35 +316,7 @@ class ClientLoginDialog(GtkBuilderDialog):
             self.dialog.destroy()
 
 
-class NoInstallerDialog(Gtk.MessageDialog):
-    MANUAL_CONF = 1
-    NEW_INSTALLER = 2
-    EXIT = 4
-
-    def __init__(self, parent=None):
-        Gtk.MessageDialog.__init__(
-            self,
-            parent,
-            0,
-            Gtk.MessageType.ERROR,
-            Gtk.ButtonsType.NONE,
-            _("Unable to install the game"),
-        )
-        self.format_secondary_text(_("No installer is available for this game"))
-        self.add_buttons(
-            _("Configure manually"),
-            self.MANUAL_CONF,
-            _("Write installer"),
-            self.NEW_INSTALLER,
-            _("Close"),
-            self.EXIT,
-        )
-        self.result = self.run()
-        self.destroy()
-
-
 class WebConnectDialog(Dialog):
-
     """Login form for external services"""
 
     def __init__(self, service, parent=None):
@@ -391,16 +339,78 @@ class WebConnectDialog(Dialog):
         self.webview = WebKit2.WebView.new_with_context(self.context)
         self.webview.load_uri(service.login_url)
         self.webview.connect("load-changed", self.on_navigation)
-        self.vbox.pack_start(self.webview, True, True, 0)
+        self.webview.connect("create", self.on_webview_popup)
+        self.vbox.pack_start(self.webview, True, True, 0)  # pylint: disable=no-member
+
+        webkit_settings = self.webview.get_settings()
+        # Allow popups (Doesn't work...)
+        webkit_settings.set_enable_write_console_messages_to_stdout(True)
+        webkit_settings.set_allow_modal_dialogs(True)
+        # Enable developer options for troubleshooting (Can be disabled in
+        # releases)
+        webkit_settings.set_javascript_can_open_windows_automatically(True)
+        webkit_settings.set_enable_developer_extras(True)
 
         self.show_all()
 
+    def enable_inspector(self):
+        """If you want a full blown Webkit inspector, call this"""
+        inspector = self.webview.get_inspector()
+        inspector.show()
+
     def on_navigation(self, widget, load_event):
         if load_event == WebKit2.LoadEvent.FINISHED:
-            uri = widget.get_uri()
-            if uri.startswith(self.service.redirect_uri):
-                self.service.request_token(uri)
+            url = widget.get_uri()
+            if url.startswith(self.service.redirect_uri):
+                self.service.request_token(url)
                 self.destroy()
+        return True
+
+    def on_webview_popup(self, widget, navigation_action):
+        """Handles web popups created by this dialog's webview"""
+        uri = navigation_action.get_request().get_uri()
+        view = WebKit2.WebView.new_with_related_view(widget)
+        view.load_uri(uri)
+        popup_dialog = WebPopupDialog(view, parent=self)
+        popup_dialog.set_modal(True)
+        popup_dialog.show()
+        return view
+
+
+class WebPopupDialog(Dialog):
+    """Dialog for handling web popups"""
+
+    def __init__(self, webview, parent=None):
+        # pylint: disable=no-member
+        self.parent = parent
+        super(WebPopupDialog, self).__init__(title=_('Loading...'), parent=parent)
+        self.webview = webview
+        self.webview.connect("ready-to-show", self.on_ready_webview)
+        self.webview.connect("notify::title", self.on_available_webview_title)
+        self.webview.connect("create", self.on_new_webview_popup)
+        self.webview.connect("close", self.on_webview_close)
+        self.vbox.pack_start(self.webview, True, True, 0)
+        self.set_border_width(0)
+        self.set_default_size(390, 500)
+
+    def on_ready_webview(self, webview):
+        self.show_all()
+
+    def on_available_webview_title(self, webview, gparamstring):
+        self.set_title(webview.get_title())
+
+    def on_new_webview_popup(self, webview, navigation_action):
+        """Handles web popups created by this dialog's webview"""
+        uri = navigation_action.get_request().get_uri()
+        view = WebKit2.WebView.new_with_related_view(webview)
+        view.load_uri(uri)
+        dialog = WebPopupDialog(view, parent=self)
+        dialog.set_modal(True)
+        dialog.show()
+        return view
+
+    def on_webview_close(self, webview):
+        self.destroy()
 
 
 class InstallerSourceDialog(Gtk.Dialog):
@@ -446,7 +456,7 @@ class DontShowAgainDialog(Gtk.MessageDialog):
         parent=None,
         checkbox_message=None,
     ):
-
+        # pylint: disable=no-member
         if settings.read_setting(setting) == "True":
             logger.info("Dialog %s dismissed by user", setting)
             return
