@@ -1,19 +1,16 @@
 """Wine runner"""
-
-# Standard Library
 # pylint: disable=too-many-lines
 import os
 import shlex
 import shutil
 from gettext import gettext as _
 
-# Lutris Modules
 from lutris import runtime
 from lutris.exceptions import GameConfigError
 from lutris.gui.dialogs import FileDialog
 from lutris.runners.commands.wine import (  # noqa: F401 pylint: disable=unused-import
-    create_prefix, delete_registry_key, eject_disc, install_cab_component, set_regedit, set_regedit_file, winecfg,
-    wineexec, winekill, winetricks
+    create_prefix, delete_registry_key, eject_disc, install_cab_component, open_wine_terminal, set_regedit,
+    set_regedit_file, winecfg, wineexec, winekill, winetricks
 )
 from lutris.runners.runner import Runner
 from lutris.settings import RUNTIME_DIR
@@ -25,7 +22,7 @@ from lutris.util.jobs import thread_safe_call
 from lutris.util.log import logger
 from lutris.util.strings import parse_version, split_arguments
 from lutris.util.wine import dxvk, nine
-from lutris.util.wine.prefix import WinePrefixManager
+from lutris.util.wine.prefix import WinePrefixManager, find_prefix
 from lutris.util.wine.wine import (
     POL_PATH, WINE_DIR, WINE_PATHS, detect_arch, display_vulkan_error, esync_display_limit_warning,
     esync_display_version_warning, fsync_display_support_warning, fsync_display_version_warning, get_default_version,
@@ -34,7 +31,7 @@ from lutris.util.wine.wine import (
 )
 from lutris.util.wine.x360ce import X360ce
 
-MIN_SAFE_VERSION = "4.0"  # Wine installers must run with at least this version
+MIN_SAFE_VERSION = "5.0"  # Wine installers must run with at least this version
 
 
 class wine(Runner):
@@ -59,12 +56,9 @@ class wine(Runner):
             "validator": shlex.split
         },
         {
-            "option":
-            "working_dir",
-            "type":
-            "directory_chooser",
-            "label":
-            _("Working directory"),
+            "option": "working_dir",
+            "type": "directory_chooser",
+            "label": _("Working directory"),
             "help": _(
                 "The location where the game is run from.\n"
                 "By default, Lutris uses the directory of the "
@@ -72,12 +66,9 @@ class wine(Runner):
             ),
         },
         {
-            "option":
-            "prefix",
-            "type":
-            "directory_chooser",
-            "label":
-            _("Wine prefix"),
+            "option": "prefix",
+            "type": "directory_chooser",
+            "label": _("Wine prefix"),
             "help": _(
                 'The prefix (also named "bottle") used by Wine.\n'
                 "It's a directory containing a set of files and "
@@ -261,12 +252,12 @@ class wine(Runner):
             },
             {
                 "option": "fsync",
-                "label": "Enable Fsync",
+                "label": _("Enable Fsync"),
                 "type": "extended_bool",
                 "callback": fsync_support_callback,
                 "callback_on": True,
                 "active": True,
-                "help": (
+                "help": _(
                     "Enable futex-based synchronization (fsync). "
                     "This will increase performance in applications "
                     "that take advantage of multi-core processors. "
@@ -563,7 +554,8 @@ class wine(Runner):
         if "Proton" not in self.get_version():
             menu_entries.append(("winecfg", _("Wine configuration"), self.run_winecfg))
         menu_entries += [
-            ("wineconsole", _("Wine console"), self.run_wineconsole),
+            ("wineshell", _("Open Bash terminal"), self.run_wine_terminal),
+            ("wineconsole", _("Open Wine console"), self.run_wineconsole),
             ("wine-regedit", _("Wine registry"), self.run_regedit),
             ("winekill", _("Kill all wine processes"), self.run_winekill),
             ("winetricks", _("Winetricks"), self.run_winetricks),
@@ -574,13 +566,15 @@ class wine(Runner):
     @property
     def prefix_path(self):
         """Return the absolute path of the Wine prefix"""
-        _prefix_path = self.game_config.get("prefix")
+        _prefix_path = self.game_config.get("prefix") \
+            or os.environ.get("WINEPREFIX") \
+            or find_prefix(self.game_exe)
         if not _prefix_path:
             logger.warning(
-                "Wine prefix not provided, defaulting to $WINEPREFIX then ~/.wine."
+                "Wine prefix not provided, defaulting to ~/.wine."
                 " This is probably not the intended behavior."
             )
-            _prefix_path = os.environ.get("WINEPREFIX") or "~/.wine"
+            _prefix_path = "~/.wine"
         return os.path.expanduser(_prefix_path)
 
     @property
@@ -750,6 +744,15 @@ class wine(Runner):
         self.prelaunch()
         self._run_executable("regedit")
 
+    def run_wine_terminal(self, *args):
+        terminal = self.system_config.get("terminal_app")
+        open_wine_terminal(
+            terminal=terminal,
+            wine_path=self.get_executable(),
+            prefix=self.prefix_path,
+            env=self.get_env(os_env=True)
+        )
+
     def run_winetricks(self, *args):
         """Run winetricks in the current context"""
         self.prelaunch()
@@ -807,7 +810,7 @@ class wine(Runner):
         if version.lower() != "manual":
             if enable:
                 if not dxvk_manager.is_available():
-                    logger.info("DXVK %s is not available yet, downloading...")
+                    logger.info("DXVK %s is not available yet, downloading...", version)
                     dxvk_manager.download()
                 dxvk_manager.enable()
             else:
@@ -868,12 +871,9 @@ class wine(Runner):
         if not isinstance(overrides, dict):
             logger.warning("DLL overrides is not a mapping: %s", overrides)
             overrides = {}
-
-        overrides = overrides.copy()
-        overrides.update(self.dll_overrides)
         return overrides
 
-    def get_env(self, os_env=True):
+    def get_env(self, os_env=False):
         """Return environment variables used by the game"""
         # Always false to runner.get_env, the default value
         # of os_env is inverted in the wine class,
@@ -895,15 +895,16 @@ class wine(Runner):
         if not ("WINEFSYNC" in env and env["WINEFSYNC"] == "1"):
             env["WINEFSYNC"] = "1" if self.runner_config.get("fsync") else "0"
 
-        # On AMD mimic, the video memory management behavior of Windows DX12
+        # On AMD, mimic the video memory management behavior of Windows DX12
         # drivers more closely, otherwise d3d12 games will crash and have other
         # funky issues.
-        if self.runner_config.get("vkd3d") and drivers.is_amd():
+        if self.runner_config.get("dxvk") and drivers.is_amd():
             env["RADV_DEBUG"] = "zerovram"
 
         overrides = self.get_dll_overrides()
         if overrides:
-            env["WINEDLLOVERRIDES"] = get_overrides_env(overrides)
+            self.dll_overrides.update(overrides)
+        env["WINEDLLOVERRIDES"] = get_overrides_env(self.dll_overrides)
         return env
 
     def get_runtime_env(self):
